@@ -1,23 +1,17 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
-  This file is part of GNU ghostscript
-
-  GNU ghostscript is free software; you can redistribute it and/or
-  modify it under the terms of the version 2 of the GNU General Public
-  License as published by the Free Software Foundation.
-
-  GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along with
-  ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
-  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfk.c,v 1.8 2008/03/23 15:27:38 Arabidopsis Exp $ */
+/* $Id: gdevpdfk.c,v 1.2 2010/07/10 22:02:18 Arabidopsis Exp $ */
 /* Lab and ICCBased color space writing */
 #include "math_.h"
 #include "memory_.h"
@@ -261,8 +255,8 @@ pdf_make_iccbased(gx_device_pdf *pdev, cos_array_t *pca, int ncomps,
     case gs_color_space_index_DeviceCMYK:
 	break;			/* implicit (default) */
     default:
-	if ((code = pdf_color_space(pdev, &v, NULL, pcs_alt,
-				    &pdf_color_space_names, false)) < 0 ||
+	if ((code = pdf_color_space_named(pdev, &v, NULL, pcs_alt,
+				    &pdf_color_space_names, false, NULL, 0)) < 0 ||
 	    (code = cos_dict_put_c_key(cos_stream_dict(pcstrm), "/Alternate",
 				       &v)) < 0
 	    )
