@@ -1,23 +1,17 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
-  This file is part of GNU ghostscript
-
-  GNU ghostscript is free software; you can redistribute it and/or
-  modify it under the terms of the version 2 of the GNU General Public
-  License as published by the Free Software Foundation.
-
-  GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along with
-  ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
-  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevxini.c,v 1.11 2008/05/04 21:04:04 Arabidopsis Exp $ */
+/* $Id: gdevxini.c,v 1.2 2010/07/10 22:02:24 Arabidopsis Exp $ */
 /* X Windows driver initialization/finalization */
 #include "memory_.h"
 #include "x_.h"
@@ -918,18 +912,6 @@ gdev_x_put_params(gx_device * dev, gs_param_list * plist)
 	    gs_closedevice(dev);
 	xdev->pwin = (Window) pwin;
     }
-    /* Do not change the image size set by Ghostview */
-    /* This gives the Ghostview user control over the /setpage entry */
-    if (xdev->is_open && xdev->ghostview) {
-	dev->width = values.width;
-	dev->height = values.height;
-	dev->x_pixels_per_inch = values.x_pixels_per_inch;
-	dev->y_pixels_per_inch = values.y_pixels_per_inch;
-	dev->HWResolution[0] = values.HWResolution[0];
-	dev->HWResolution[1] = values.HWResolution[1];
-	dev->MediaSize[0] = values.MediaSize[0];
-	dev->MediaSize[1] = values.MediaSize[1];
-    }
     /* Restore the original page size if it was set by Ghostview */
     /* This gives the Ghostview user control over the /setpage entry */
     if (xdev->is_open && xdev->ghostview) {
