@@ -1,23 +1,17 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
-  This file is part of GNU ghostscript
-
-  GNU ghostscript is free software; you can redistribute it and/or
-  modify it under the terms of the version 2 of the GNU General Public
-  License as published by the Free Software Foundation.
-
-  GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along with
-  ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
-  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsimage.c,v 1.13 2009/04/19 13:54:23 Arabidopsis Exp $ */
+/* $Id: gsimage.c,v 1.2 2010/07/10 22:02:26 Arabidopsis Exp $ */
 /* Image setup procedures for Ghostscript library */
 #include "memory_.h"
 #include "math_.h"
@@ -182,12 +176,12 @@ is_image_visible(const gs_image_common_t * pic, gs_state * pgs, gx_clip_path *pc
             mat.tx = ctm_only(pgs).tx - pic->ImageMatrix.tx;
             mat.ty = ctm_only(pgs).ty - pic->ImageMatrix.ty;
         } else {
-	code = gs_matrix_invert(&pic->ImageMatrix, &mat);
-	if (code < 0)
-	    return code;
-	code = gs_matrix_multiply(&mat, &ctm_only(pgs), &mat);
-	if (code < 0)
-	    return code;
+            code = gs_matrix_invert(&pic->ImageMatrix, &mat);
+	    if (code < 0)
+	        return code;
+	    code = gs_matrix_multiply(&mat, &ctm_only(pgs), &mat);
+	    if (code < 0)
+	        return code;
         }
 	code = gs_bbox_transform(&image_rect, &mat, &device_rect);
 	if (code < 0)
