@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gxblend1.c 9308 2008-12-29 06:43:42Z mvrhel $ */
+/* $Id: gxblend1.c,v 1.2 2010/07/10 22:02:21 Arabidopsis Exp $ */
 /* PDF 1.4 blending functions */
 
 #include "memory_.h"
@@ -24,6 +24,7 @@
 #include "gdevdevn.h"
 #include "gdevp14.h"
 #include "vdtrace.h"
+#include "gxdcconv.h"
 
 #ifdef DUMP_TO_PNG
 #include "png_.h"
@@ -32,8 +33,7 @@
 /*
  * Unpack a device color.  This routine is similar to the device's
  * decode_color procedure except for two things.  The procedure produces 1
- * byte values instead of gx_color_values (2 bytes) and the output values
- * are inverted for subtractive color spaces (like CMYK).  A separate
+ * byte values instead of gx_color_values (2 bytes).  A separate
  * procedure is used instead of the decode_color to minimize execution time.
  */
 void
@@ -138,11 +138,14 @@ pdf14_unpack_custom(int num_comp, gx_color_index color,
 	out[i] = 0xff - gx_color_value_to_byte(cm_values[i]);
 }
 
-
+#if RAW_DUMP
+extern unsigned int global_index;
+#endif
 
 void
 pdf14_preserve_backdrop(pdf14_buf *buf, pdf14_buf *tos, bool has_shape)
 {
+
     /* make copy of backdrop for compositing */
     int x0 = max(buf->rect.p.x, tos->rect.p.x);
     int x1 = min(buf->rect.q.x, tos->rect.q.x);
@@ -154,7 +157,8 @@ pdf14_preserve_backdrop(pdf14_buf *buf, pdf14_buf *tos, bool has_shape)
 	byte *buf_plane = buf->data + x0 - buf->rect.p.x + (y0 - buf->rect.p.y) * buf->rowstride;
 	byte *tos_plane = tos->data + x0 - tos->rect.p.x + (y0 - tos->rect.p.y) * tos->rowstride;
 	int i;
-	int n_chan_copy = buf->n_chan + (tos->has_shape ? 1 : 0);
+	/*int n_chan_copy = buf->n_chan + (tos->has_shape ? 1 : 0);*/
+	int n_chan_copy = tos->n_chan + (tos->has_shape ? 1 : 0);
 
 	for (i = 0; i < n_chan_copy; i++) {
 		byte *buf_ptr = buf_plane;
@@ -172,8 +176,29 @@ pdf14_preserve_backdrop(pdf14_buf *buf, pdf14_buf *tos, bool has_shape)
 	if (has_shape && !tos->has_shape)
 	    memset (buf_plane, 0, buf->planestride);
     }
+
+
+#if RAW_DUMP
+
+    if (x0 < x1 && y0 < y1) {
+        
+        byte *buf_plane = buf->data + x0 - buf->rect.p.x + 
+            (y0 - buf->rect.p.y) * buf->rowstride;
+
+        dump_raw_buffer(y1-y0, x1 - x0, buf->n_planes,
+                    buf->planestride, buf->rowstride, 
+                    "BackDropInit",buf_plane);
+
+        global_index++;
+
+    }
+
+#endif
+
 }
 
+
+
 void
 pdf14_compose_group(pdf14_buf *tos, pdf14_buf *nos, pdf14_buf *maskbuf, 
 	      int x0, int x1, int y0, int y1, int n_chan, bool additive,
@@ -194,7 +219,7 @@ pdf14_compose_group(pdf14_buf *tos, pdf14_buf *nos, pdf14_buf *maskbuf,
     byte mask_bg_alpha = 0; /* Quiet compiler. */
     int width = x1 - x0;
     int x, y;
-    int i;
+    int i, tmp;
     byte tos_pixel[PDF14_MAX_PLANES];
     byte nos_pixel[PDF14_MAX_PLANES];
     bool tos_isolated = tos->isolated;
@@ -206,6 +231,9 @@ pdf14_compose_group(pdf14_buf *tos, pdf14_buf *nos, pdf14_buf *maskbuf,
     int nos_shape_offset = n_chan * nos_planestride;
     bool nos_has_shape = nos->has_shape;
     byte *mask_tr_fn = NULL; /* Quiet compiler. */
+#if RAW_DUMP
+    byte *composed_ptr = NULL;
+#endif
 
     rect_merge(nos->bbox, tos->bbox);
 
@@ -216,14 +244,58 @@ pdf14_compose_group(pdf14_buf *tos, pdf14_buf *nos, pdf14_buf *maskbuf,
     else
 	nos_alpha_g_ptr = NULL;
 
-    if (maskbuf != NULL) {
-	mask_ptr = maskbuf->data + x0 - maskbuf->rect.p.x +
-		(y0 - maskbuf->rect.p.y) * maskbuf->rowstride;
-	mask_planestride = maskbuf->planestride;
-	mask_bg_alpha = maskbuf->alpha;
-	mask_tr_fn = maskbuf->transfer_fn;
+    if (maskbuf != NULL) { 
+       
+        mask_tr_fn = maskbuf->transfer_fn;
+
+        if (maskbuf->data != NULL) {
+
+	    mask_ptr = maskbuf->data + x0 - maskbuf->rect.p.x +
+		    (y0 - maskbuf->rect.p.y) * maskbuf->rowstride;
+            mask_planestride = maskbuf->planestride;
+
+        } else {
+
+            /* We may have a case, where we are outside the maskbuf rect. */
+            /* We would have avoided creating the maskbuf->data */
+            /* In that case, we should use the background alpha value */
+            /* See discussion on the BC entry in the PDF spec */
+
+            mask_bg_alpha = maskbuf->alpha;
+
+            /* Adjust alpha by the mask background alpha */
+
+            mask_bg_alpha = mask_tr_fn[mask_bg_alpha];
+            tmp = alpha * mask_bg_alpha + 0x80;
+            alpha = (tmp + (tmp >> 8)) >> 8;
+
+        }
+
+
+    } 
+
+
+#if RAW_DUMP
+
+    composed_ptr = nos_ptr;
+
+    dump_raw_buffer(y1-y0, width, tos->n_planes,
+            tos_planestride, tos->rowstride, 
+            "ImageTOS",tos_ptr);
+
+    dump_raw_buffer(y1-y0, width, nos->n_planes,
+                nos_planestride, nos->rowstride, 
+                "ImageNOS",nos_ptr);
+
+    if(mask_ptr != NULL){
+
+        dump_raw_buffer(y1-y0, width, maskbuf->n_planes,
+                        maskbuf->planestride, maskbuf->rowstride, 
+                        "Mask",mask_ptr);
     }
 
+#endif
+
     for (y = y0; y < y1; ++y) {
 	for (x = 0; x < width; ++x) {
 	    byte pix_alpha = alpha;
@@ -244,35 +316,16 @@ pdf14_compose_group(pdf14_buf *tos, pdf14_buf *nos, pdf14_buf *maskbuf,
 	    }
 
 	    if (mask_ptr != NULL) {
-		int mask_alpha = mask_ptr[x + num_comp * mask_planestride];
-		int tmp;
-		byte mask;
 
-		    /*
-		    * The mask data is really monochrome.  Thus for additive (RGB)
-		    * we use the R channel for alpha since R = G = B.  For
-		    * subtractive (CMYK) we use the K channel.
-		    */
-		if (mask_alpha == 255) {
-		    /* todo: rgba->mask */
-		    mask = additive ? mask_ptr[x]
-				    : 255 - mask_ptr[x + 3 * mask_planestride];
-		} else if (mask_alpha == 0)
-		    mask = mask_bg_alpha;
-		else {
-		    int t2 = additive ? mask_ptr[x]
-				    : 255 - mask_ptr[x + 3 * mask_planestride];
+                byte mask = mask_ptr[x];
 
-		    t2 = (t2 - mask_bg_alpha) * mask_alpha + 0x80;
-		    mask = mask_bg_alpha + ((t2 + (t2 >> 8)) >> 8);
-		}
 		mask = mask_tr_fn[mask];
 		tmp = pix_alpha * mask + 0x80;
 		pix_alpha = (tmp + (tmp >> 8)) >> 8;
 #		    if VD_PAINT_MASK
 		    vd_pixel(int2fixed(x), int2fixed(y), mask);
 #		    endif
-	    }
+            } 
 
 	    if (nos_knockout) {
 		byte *nos_shape_ptr = nos_has_shape ?
@@ -332,8 +385,61 @@ pdf14_compose_group(pdf14_buf *tos, pdf14_buf *nos, pdf14_buf *maskbuf,
 	if (mask_ptr != NULL)
 	    mask_ptr += maskbuf->rowstride;
     }
+
+
+    /* Lets look at composed result */
+
+
+#if RAW_DUMP
+
+	/* The group alpha should disappear */
+    dump_raw_buffer(y1-y0, width, tos->n_planes - tos->has_alpha_g,
+                nos_planestride, nos->rowstride, 
+                "Composed",composed_ptr);
+
+    global_index++;
+
+
+#endif
+
 }
 
+/*
+ * Encode a list of smask colorant values into a gx_color_index_value.
+ * This has its own encoder as it may have a different number of colorants
+ * compared to the actual device.
+ */
+gx_color_index
+pdf14_encode_smask_color(gx_device *dev, const gx_color_value	colors[],int ncomp)
+{
+    int drop = sizeof(gx_color_value) * 8 - 8;
+    gx_color_index color = 0;
+    int i;
+
+    for (i = 0; i < ncomp; i++) {
+	color <<= 8;
+	color |= (colors[i] >> drop);
+    }
+    return (color == gx_no_color_index ? color ^ 1 : color);
+}
+
+/*
+ * Decode a gx_color_index value back to a list of colorant values.
+  * This has its own decoder as it may have a different number of colorants
+ * compared to the actual device.*/
+int
+pdf14_decode_smask_color(gx_device * dev, gx_color_index color, gx_color_value * out, int ncomp)
+{
+    int i;
+
+    for (i = 0; i < ncomp; i++) {
+	out[ncomp - i - 1] = (gx_color_value) ((color & 0xff) * 0x101);
+	color >>= 8;
+    }
+    return 0;
+}
+
+
 /*
  * Encode a list of colorant values into a gx_color_index_value.
  */
@@ -376,8 +482,20 @@ pdf14_decode_color(gx_device * dev, gx_color_index color, gx_color_value * out)
 gx_color_index
 pdf14_compressed_encode_color(gx_device *dev, const gx_color_value colors[])
 {
-    return devn_encode_compressed_color(dev, colors,
-		    &(((pdf14_device *)dev)->devn_params));
+    gs_devn_params *pdevn_params = NULL;
+
+    if (dev->procs.ret_devn_params != NULL)
+	pdevn_params = dev_proc(dev, ret_devn_params)(dev);
+    /* If there was no dev_proc or it returned NULL, assume pdf14 device devn_params */
+    if (pdevn_params == NULL) {
+#ifdef DEBUG
+	if (strncmp(dev->dname, "pdf14", 5))
+	    eprintf1("pdf14_compressed_encode_color devn_params not from pdf14 device, device = '%s'\n",
+			dev->dname);
+#endif
+    	pdevn_params = &(((pdf14_device *)dev)->devn_params);
+    }
+    return devn_encode_compressed_color(dev, colors, pdevn_params);
 }
 
 /*
@@ -389,8 +507,20 @@ int
 pdf14_compressed_decode_color(gx_device * dev, gx_color_index color,
 	       						gx_color_value * out)
 {
-    return devn_decode_compressed_color(dev, color, out,
-		    &(((pdf14_device *)dev)->devn_params));
+    gs_devn_params *pdevn_params = NULL;
+
+    if (dev->procs.ret_devn_params != NULL)
+	pdevn_params = dev_proc(dev, ret_devn_params)(dev);
+    /* If there was no dev_proc or it returned NULL, assume pdf14 device devn_params */
+    if (pdevn_params == NULL) {
+#ifdef DEBUG
+	if (strncmp(dev->dname, "pdf14", 5))
+	    eprintf1("pdf14_compressed_decode_color devn_params not from pdf14 device, device = '%s'\n",
+			dev->dname);
+#endif
+    	pdevn_params = &(((pdf14_device *)dev)->devn_params);
+    }
+    return devn_decode_compressed_color(dev, color, out, pdevn_params);
 }
 
 void
