@@ -1,23 +1,17 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
-  This file is part of GNU ghostscript
-
-  GNU ghostscript is free software; you can redistribute it and/or
-  modify it under the terms of the version 2 of the GNU General Public
-  License as published by the Free Software Foundation.
-
-  GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along with
-  ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
-  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxfcopy.c,v 1.12 2008/03/23 15:27:57 Arabidopsis Exp $ */
+/* $Id: gxfcopy.c,v 1.2 2010/07/10 22:02:25 Arabidopsis Exp $ */
 /* Font copying for high-level output */
 #include "memory_.h"
 #include "gx.h"
@@ -1982,7 +1976,11 @@ gs_copy_font(gs_font *font, const gs_matrix *orig_matrix, gs_memory_t *mem, gs_f
     memset(&info, 0, sizeof(info));
     info.Flags_requested = ~0;
     code = font->procs.font_info(font, NULL, ~0, &info);
-    if (code < 0)
+
+    /* We can ignore a lack of FontInfo for TrueType fonts which 
+     * are descendants of CID fonts
+     */
+    if (code < 0 && !(font->FontType == ft_CID_TrueType))
 	return code;
 
     /* Allocate the generic copied information. */
@@ -2432,3 +2430,4 @@ copied_get_notdef(const gs_font *font)
 
     return cfdata->notdef;
 }
+
