@@ -1,23 +1,17 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
-  This file is part of GNU ghostscript
-
-  GNU ghostscript is free software; you can redistribute it and/or
-  modify it under the terms of the version 2 of the GNU General Public
-  License as published by the Free Software Foundation.
-
-  GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along with
-  ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
-  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gximag3x.c,v 1.13 2009/04/19 13:54:26 Arabidopsis Exp $ */
+/* $Id: gximag3x.c,v 1.2 2010/07/10 22:02:20 Arabidopsis Exp $ */
 /* ImageType 3x image implementation */
 /****** THE REAL WORK IS NYI ******/
 #include "math_.h"		/* for ceil, floor */
@@ -605,8 +599,20 @@ gx_image3x_plane_data(gx_image_enum_common_t * info,
     h = min(height, h1);
     /* Initialized rows_used in case we get an error. */
     *rows_used = 0;
+
+/* FIX_ME.  In the soft mask branch
+   the file associated with bug 689290
+   gets into an endless loop.  This
+   is related to the size mismatch of the 
+   soft mask which is 1x1 and the 
+   image size */
+
     if (h <= 0)
+#ifdef ENDLESS_LOOP     
 	return 0;
+#else
+        return 1;
+#endif
 
     /* Handle masks from separate sources. */
     for (i = 0; i < NUM_MASKS; ++i)
