@@ -1,23 +1,17 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
-  This file is part of GNU ghostscript
-
-  GNU ghostscript is free software; you can redistribute it and/or
-  modify it under the terms of the version 2 of the GNU General Public
-  License as published by the Free Software Foundation.
-
-  GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along with
-  ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
-  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: stream.c,v 1.11 2008/03/23 15:28:04 Arabidopsis Exp $ */
+/* $Id: stream.c,v 1.2 2010/07/10 22:02:24 Arabidopsis Exp $ */
 /* Stream package for Ghostscript interpreter */
 #include "stdio_.h"		/* includes std.h */
 #include "memory_.h"
@@ -138,6 +132,7 @@ s_init_state(stream_state *st, const stream_template *template,
     st->memory = mem;
     st->report_error = s_no_report_error;
     st->min_left = 0;
+    st->error_string[0] = 0;
 }
 stream_state *
 s_alloc_state(gs_memory_t * mem, gs_memory_type_ptr_t stype,
@@ -1265,9 +1260,9 @@ s_close_disable(stream *s)
 int
 file_close_disable(stream * s)
 {
-    int code = (*s->save_close)(s);
-
-    if (code)
+    int code;
+    
+    if ((*s->save_close != NULL) && ((code = (*s->save_close)(s)) != 0))
 	return code;
     s_close_disable(s);
     return file_close_finish(s);
