@@ -1,24 +1,21 @@
 /*
     jbig2dec
-    
+
     Copyright (C) 2001-2005 Artifex Software, Inc.
-    
+
     This software is distributed under license and may not
     be copied, modified or distributed except as expressly
     authorized under the terms of the license contained in
     the file LICENSE in this distribution.
-                                                                                
-    For information on commercial licensing, go to
-    http://www.artifex.com/licensing/ or contact
-    Artifex Software, Inc.,  101 Lucas Valley Road #110,
-    San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 
-    $Id: jbig2_page.c,v 1.8 2009/04/19 13:54:46 Arabidopsis Exp $
+    For further licensing information refer to http://artifex.com/ or
+    contact Artifex Software, Inc., 7 Mt. Lassen Drive - Suite A-134,
+    San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif 
+#endif
 #include "os_types.h"
 
 #include <stdlib.h>
@@ -57,14 +54,14 @@ dump_page_info(Jbig2Ctx *ctx, Jbig2Segment *segment, Jbig2Page *page)
 }
 
 /**
- * jbig2_read_page_info: parse page info segment
+ * jbig2_page_info: parse page info segment
  *
- * parse the page info segment data and fill out a corresponding
- * Jbig2Page struct is returned, including allocating an image
- * buffer for the page (or the first stripe)
+ * Parse the page info segment data and fill out a corresponding
+ * Jbig2Page struct and ready it for subsequent rendered data,
+ * including allocating an image buffer for the page (or the first stripe)
  **/
 int
-jbig2_parse_page_info (Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *segment_data)
+jbig2_page_info (Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *segment_data)
 {
     Jbig2Page *page;
 
@@ -76,19 +73,18 @@ jbig2_parse_page_info (Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *segm
         jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
             "unexpected page info segment, marking previous page finished");
     }
-        
+
     /* find a free page */
     {
         int index, j;
         index = ctx->current_page;
         while (ctx->pages[index].state != JBIG2_PAGE_FREE) {
             index++;
-            if (index >= ctx->max_page_index) { /* FIXME: should also look for freed pages? */
+            if (index >= ctx->max_page_index) {
                 /* grow the list */
 		ctx->pages = jbig2_realloc(ctx->allocator, ctx->pages,
 			(ctx->max_page_index <<= 2) * sizeof(Jbig2Page));
                 for (j=index; j < ctx->max_page_index; j++) {
-                    /* note to raph: and look, it gets worse! */
                     ctx->pages[j].state = JBIG2_PAGE_FREE;
                     ctx->pages[j].number = 0;
                     ctx->pages[j].image = NULL;
@@ -101,17 +97,17 @@ jbig2_parse_page_info (Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *segm
         page->state = JBIG2_PAGE_NEW;
         page->number = segment->page_association;
     }
-    
+
     /* FIXME: would be nice if we tried to work around this */
     if (segment->data_length < 19) {
         return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
             "segment too short");
     }
-    
+
     /* 7.4.8.x */
     page->width = jbig2_get_int32(segment_data);
     page->height = jbig2_get_int32(segment_data + 4);
-    
+
     page->x_resolution = jbig2_get_int32(segment_data + 8);
     page->y_resolution = jbig2_get_int32(segment_data + 12);
     page->flags = segment_data[16];
@@ -133,12 +129,12 @@ jbig2_parse_page_info (Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *segm
         page->striped = TRUE;
     }
     page->end_row = 0;
-    
+
     if (segment->data_length > 19) {
         jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
             "extra data in segment");
     }
-    
+
     dump_page_info(ctx, segment, page);
 
     /* allocate an approprate page image buffer */
@@ -149,7 +145,6 @@ jbig2_parse_page_info (Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *segm
         page->image = jbig2_image_new(ctx, page->width, page->height);
     }
     if (page->image == NULL) {
-        jbig2_free(ctx->allocator, page);
         return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
             "failed to allocate buffer for page image");
     } else {
@@ -160,15 +155,15 @@ jbig2_parse_page_info (Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *segm
             page->image->width, page->image->height,
             page->image->stride*page->image->height);
     }
-    
+
     return 0;
 }
 
 /**
- * jbig2_parse_end_of_stripe: parse an end of stripe segment
+ * jbig2_end_of_stripe: parse and implement an end of stripe segment
  **/
 int
-jbig2_parse_end_of_stripe(Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *segment_data)
+jbig2_end_of_stripe(Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *segment_data)
 {
     Jbig2Page page = ctx->pages[ctx->current_page];
     int end_row;
@@ -177,13 +172,13 @@ jbig2_parse_end_of_stripe(Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *s
     if (end_row < page.end_row) {
 	jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number,
 	    "end of stripe segment with non-positive end row advance"
-	    "(new end row %d vs current end row %d)", 
+	    " (new end row %d vs current end row %d)",
 	    end_row, page.end_row);
     } else {
 	jbig2_error(ctx, JBIG2_SEVERITY_INFO, segment->number,
 	    "end of stripe: advancing end row to %d", end_row);
     }
-    
+
     page.end_row = end_row;
 
     return 0;
@@ -194,7 +189,7 @@ jbig2_parse_end_of_stripe(Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *s
  *
  * called upon seeing an 'end of page' segment, this routine
  * marks a page as completed so it can be returned.
- * compositing will have already happened in the previous 
+ * compositing will have already happened in the previous
  * segment handlers.
  **/
 int
@@ -224,10 +219,10 @@ jbig2_complete_page (Jbig2Ctx *ctx)
 }
 
 /**
- * jbig2_parse_end_of_page: parse an end of page segment
+ * jbig2_end_of_page: parse and implement an end of page segment
  **/
 int
-jbig2_parse_end_of_page(Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *segment_data)
+jbig2_end_of_page(Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *segment_data)
 {
     uint32_t page_number = ctx->pages[ctx->current_page].number;
 
@@ -236,12 +231,12 @@ jbig2_parse_end_of_page(Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *seg
             "end of page marker for page %d doesn't match current page number %d",
             segment->page_association, page_number);
     }
-    
+
     jbig2_error(ctx, JBIG2_SEVERITY_INFO, segment->number,
         "end of page %d", page_number);
 
     jbig2_complete_page(ctx);
-    
+
 #ifdef OUTPUT_PBM
     jbig2_image_write_pbm(ctx->pages[ctx->current_page].image, stdout);
 #endif
@@ -269,7 +264,7 @@ jbig2_page_add_result(Jbig2Ctx *ctx, Jbig2Page *page, Jbig2Image *image,
 	    jbig2_image_resize(ctx, page->image,
 		page->image->width, new_height);
 	}
-    }    
+    }
 
     jbig2_image_compose(ctx, page->image, image,
                         x, y + page->end_row, JBIG2_COMPOSE_OR);
@@ -299,10 +294,10 @@ Jbig2Image *jbig2_page_out(Jbig2Ctx *ctx)
             ctx->pages[index].state = JBIG2_PAGE_RETURNED;
             jbig2_error(ctx, JBIG2_SEVERITY_DEBUG, -1,
                 "page %d returned to the client", ctx->pages[index].number);
-            return ctx->pages[index].image;
+            return jbig2_image_clone(ctx, ctx->pages[index].image);
         }
     }
-    
+
     /* no pages available */
     return NULL;
 }
@@ -313,18 +308,18 @@ Jbig2Image *jbig2_page_out(Jbig2Ctx *ctx)
 int jbig2_release_page(Jbig2Ctx *ctx, Jbig2Image *image)
 {
     int index;
-    
+
     /* find the matching page struct and mark it released */
     for (index = 0; index < ctx->max_page_index; index++) {
         if (ctx->pages[index].image == image) {
-            /* todo: free associated image */
+            jbig2_image_release(ctx, image);
             ctx->pages[index].state = JBIG2_PAGE_RELEASED;
             jbig2_error(ctx, JBIG2_SEVERITY_DEBUG, -1,
                 "page %d released by the client", ctx->pages[index].number);
             return 0;
         }
     }
-    
+
     /* no matching pages */
     jbig2_error(ctx, JBIG2_SEVERITY_WARNING, -1,
         "jbig2_release_page called on unknown page");
