@@ -1,23 +1,17 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
-  This file is part of GNU ghostscript
-
-  GNU ghostscript is free software; you can redistribute it and/or
-  modify it under the terms of the version 2 of the GNU General Public
-  License as published by the Free Software Foundation.
-
-  GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along with
-  ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
-  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-// $Id: dwuninst.cpp,v 1.10 2007/09/11 15:24:41 Arabidopsis Exp $
+// $Id: dwuninst.cpp,v 1.2 2010/07/10 22:02:41 Arabidopsis Exp $
 
 #define STRICT
 #include <windows.h>
@@ -848,7 +842,7 @@ init(void)
 
     if (strlen(szLogFile) == 0) {
 	    message_box("Usage: uninstgs logfile.txt [-q]", 
-		    "GNU Ghostscript Uninstall", MB_OK);
+		    "GPL Ghostscript Uninstall", MB_OK);
 	    return FALSE;
     }
 
